"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.build_docs_project = void 0;
const luxon_1 = require("luxon");
const env_1 = require("../env");
const themes_1 = require("../themes");
const template_1 = require("../template");
const hash_1 = require("../hash");
const fs_1 = require("../fs");
const raw_1 = require("./raw");
const mustache_1 = require("./mustache");
const markdown_1 = require("./markdown");
const jsonschema_1 = require("./jsonschema");
const sitemap_1 = require("./sitemap");
const rss_1 = require("./rss");
const icalendar_1 = require("./icalendar");
const time_1 = require("../time");
async function build_docs_project(conf) {
    const now = luxon_1.DateTime.now();
    const conf_hash = (0, hash_1.hash_obj)(conf);
    const themes = await (0, themes_1.load_themes)(conf);
    const get_metadata = (0, fs_1.read_json)(conf.metadata, false).then(({ parsed }) => parsed);
    const metadata = await (0, fs_1.with_default_if_missing)(get_metadata, {
        last_build: null,
        files: {},
    });
    const state = {
        conf,
        seen_files: new Set(),
        env: (0, env_1.build_env_scope)(conf),
        layouts: Object.create(null),
        themes: themes,
        theme_groups: {
            all: Object.values(themes),
            light: [],
            dark: [],
            high_contrast: [],
            low_contrast: [],
            monochrome: [],
            greyscale: [],
            protanopia_safe: [],
            deuteranopia_safe: [],
            tritanopia_safe: [],
        },
        old_metadata: metadata,
        new_metadata: {
            last_build: {
                time: now.toISO(),
                config_hash: conf_hash,
            },
            files: {}
        },
        extras: await (0, template_1.load_extras)(),
        made_directories: new Set(),
        rss: [],
        sitemap: [],
        events: [],
        event_series: [],
        calendars: [],
        build_time: (0, time_1.as_context_time)(now),
    };
    for (const theme of Object.values(themes)) {
        for (const label of theme.labels) {
            state.theme_groups[label].push(theme);
        }
    }
    if (conf.input.raw) {
        await (0, raw_1.copy_raw_files)(state);
    }
    if (conf.input.text) {
        await (0, mustache_1.render_text_file_templates)(state);
    }
    if (conf.input.markdown) {
        await (0, markdown_1.render_markdown_files)(state);
    }
    if (conf.input['schema+json'] || conf.input['schema+yaml']) {
        await (0, jsonschema_1.render_json_schema_files)(state);
    }
    // todo: other file types...
    await (0, sitemap_1.write_sitemap_if_needed)(state);
    await (0, rss_1.write_rss_if_needed)(state);
    await (0, icalendar_1.write_events_and_calendars_if_needed)(state);
    // Write the updated metadata file
    await (0, fs_1.write_json)(conf.metadata, state.new_metadata, true);
}
exports.build_docs_project = build_docs_project;
//# sourceMappingURL=index.js.map